searchData={"content_type":"text/markdown","items":[{"doc":"","ref":"Hold.html","title":"Hold","type":"module"},{"doc":"A double ended queue (deque) implementation.\n\nDouble ended queues optimize for adding items to one end of the queue and\nremoving from the other - FIFO queues.","ref":"Hold.BankersQueue.html","title":"Hold.BankersQueue","type":"module"},{"doc":"Removes first item in `queue`.\n\nReturns `{:value, {item, queu}}` where `item` is what was removed and `queue`\nis the remaining queue. In cases where the queue is empty `{:empty, queue}`\nis retruned.\n\n```elixir\niex> queue = Hold.BankersQueue.new()\niex> queue = Hold.BankersQueue.enqueue(queue, 1)\niex> {:value, {1, queue}} = Hold.BankersQueue.dequeue(queue)\niex> Hold.BankersQueue.peek(queue)\n:empty\n```","ref":"Hold.BankersQueue.html#dequeue/1","title":"Hold.BankersQueue.dequeue/1","type":"function"},{"doc":"Add `item` to `queue`.\n\n```elixir\niex> queue = Hold.BankersQueue.new()\niex> queue = Hold.BankersQueue.enqueue(queue, 1)\niex> Hold.BankersQueue.peek(queue)\n{:value, 1}\n```","ref":"Hold.BankersQueue.html#enqueue/2","title":"Hold.BankersQueue.enqueue/2","type":"function"},{"doc":"Create a new queue from a list keepin the same order of the original list;\nthe head item of the list is the first item in the queue.\n\n```elixir\niex> queue = Hold.BankersQueue.from_list([1, 2, 3, 4, 5])\niex> Hold.BankersQueue.peek(queue)\n{:value, 1}\n```","ref":"Hold.BankersQueue.html#from_list/1","title":"Hold.BankersQueue.from_list/1","type":"function"},{"doc":"Create a new queue.","ref":"Hold.BankersQueue.html#new/0","title":"Hold.BankersQueue.new/0","type":"function"},{"doc":"","ref":"Hold.BankersQueue.html#peek/1","title":"Hold.BankersQueue.peek/1","type":"function"},{"doc":"","ref":"Hold.BankersQueue.html#t:sized_list/0","title":"Hold.BankersQueue.sized_list/0","type":"type"},{"doc":"","ref":"Hold.BankersQueue.html#t:sized_list/1","title":"Hold.BankersQueue.sized_list/1","type":"type"},{"doc":"","ref":"Hold.BankersQueue.html#t:t/0","title":"Hold.BankersQueue.t/0","type":"type"},{"doc":"","ref":"Hold.BankersQueue.html#t:t/1","title":"Hold.BankersQueue.t/1","type":"type"},{"doc":"A coordinate in an xy plane with (0,0) being the top left corner.","ref":"Hold.Coordinate.html","title":"Hold.Coordinate","type":"module"},{"doc":"Return all surrounding coordinates.\n\n```elixir\niex> Hold.Coordinate.around({1, 1})\n[{1, 0}, {1, 2}, {2, 1}, {2, 0}, {2, 2}, {0, 1}, {0, 0}, {0, 2}]\n```","ref":"Hold.Coordinate.html#around/1","title":"Hold.Coordinate.around/1","type":"function"},{"doc":"Return downward coordinate.\n\n```elixir\niex> Hold.Coordinate.down({0, 1})\n{1, 1}\n\niex> Hold.Coordinate.down({-1, 0})\n{0, 0}\n```","ref":"Hold.Coordinate.html#down/1","title":"Hold.Coordinate.down/1","type":"function"},{"doc":"Return coordinate to the left\n\n```elixir\niex> Hold.Coordinate.left({1, 1})\n{1, 0}\n\niex> Hold.Coordinate.left({0, -1})\n{0, -2}\n```","ref":"Hold.Coordinate.html#left/1","title":"Hold.Coordinate.left/1","type":"function"},{"doc":"Create a new Coordinate","ref":"Hold.Coordinate.html#new/2","title":"Hold.Coordinate.new/2","type":"function"},{"doc":"Return coordinate to the right\n\n```elixir\niex> Hold.Coordinate.right({1, 1})\n{1, 2}\n\niex> Hold.Coordinate.right({0, -1})\n{0, 0}\n```","ref":"Hold.Coordinate.html#right/1","title":"Hold.Coordinate.right/1","type":"function"},{"doc":"Return upward coordinate.\n\n```elixir\niex> Hold.Coordinate.up({0, 1})\n{-1, 1}\n```","ref":"Hold.Coordinate.html#up/1","title":"Hold.Coordinate.up/1","type":"function"},{"doc":"","ref":"Hold.Coordinate.html#t:t/0","title":"Hold.Coordinate.t/0","type":"type"},{"doc":"A collection of values held at `Hold.Coordinate`s.","ref":"Hold.CoordinateGrid.html","title":"Hold.CoordinateGrid","type":"module"},{"doc":"Add `value` to `grid` at `coord`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> Hold.CoordinateGrid.get(grid, {1, 1})\n{{1, 1}, :mine}\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> Hold.CoordinateGrid.get(grid, {10, 10})\nnil\n```","ref":"Hold.CoordinateGrid.html#add/3","title":"Hold.CoordinateGrid.add/3","type":"function"},{"doc":"Get value to downward from `coord` in `grid`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {2, 1}, :powerup)\niex> Hold.CoordinateGrid.down(grid, {1, 1})\n{{2, 1}, :powerup}\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {2, 1}, :powerup)\niex> Hold.CoordinateGrid.right(grid, {2, 1})\nnil\n```","ref":"Hold.CoordinateGrid.html#down/2","title":"Hold.CoordinateGrid.down/2","type":"function"},{"doc":"Returns whether `grid` contains a coord to the downward from `coord`.\n\nAn optional predicate function can be passed as an additional test on found\nvalue.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {2, 1}, :powerup)\niex> Hold.CoordinateGrid.down?(grid, {1, 1})\ntrue\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {2, 1}, :powerup)\niex> Hold.CoordinateGrid.down?(grid, {2, 1})\nfalse\n```\n\nAn optional predicate can be passed to further test found `t:cell`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {2, 1}, :powerup)\niex> Hold.CoordinateGrid.down?(grid, {1, 1}, fn {_coord, value} -> value == :powerup end)\ntrue\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {2, 1}, :powerup)\niex> Hold.CoordinateGrid.down?(grid, {1, 1}, fn {_coord, value} -> value == :mine end)\nfalse\n```","ref":"Hold.CoordinateGrid.html#down?/3","title":"Hold.CoordinateGrid.down?/3","type":"function"},{"doc":"Get value of `grid` at `coord` returning nil in the case of an empty\n`t:cell`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> Hold.CoordinateGrid.get(grid, {1, 1})\n{{1, 1}, :mine}\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> Hold.CoordinateGrid.get(grid, {10, 10})\nnil\n```","ref":"Hold.CoordinateGrid.html#get/2","title":"Hold.CoordinateGrid.get/2","type":"function"},{"doc":"Get value to left of `coord` in `grid`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.left(grid, {1, 2})\n{{1, 1}, :mine}\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.left(grid, {1, 1})\nnil\n```","ref":"Hold.CoordinateGrid.html#left/2","title":"Hold.CoordinateGrid.left/2","type":"function"},{"doc":"Returns whether `grid` contains a coord to the immediate left of `coord`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.left?(grid, {1, 2})\ntrue\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.left?(grid, {1, 1})\nfalse\n```\n\nAn optional predicate can be passed to further test found `t:cell`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.left?(grid, {1, 2}, fn {_coord, value} -> value == :mine end)\ntrue\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.left?(grid, {1, 2}, fn {_coord, value} -> value != :mine end)\nfalse\n```","ref":"Hold.CoordinateGrid.html#left?/3","title":"Hold.CoordinateGrid.left?/3","type":"function"},{"doc":"Return whether `t:coord` contains a `t:cell` in `grid`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> Hold.CoordinateGrid.member?(grid, {1, 1})\ntrue\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> Hold.CoordinateGrid.member?(grid, {10, 10})\nfalse\n\n```\n\nAn optional predicate can be passed to further test found `t:cell`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> Hold.CoordinateGrid.member?(grid, {1, 1}, fn {_coord, value} -> value != :mine end)\nfalse\n```","ref":"Hold.CoordinateGrid.html#member?/3","title":"Hold.CoordinateGrid.member?/3","type":"function"},{"doc":"Return list of immediate neighboring coordinates to `coord` members of `grid`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {2, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.neighbors(grid, {1, 1})\n[{{2, 1}, :mine}, {{1, 2}, :powerup}]\n```\n\nAn optional predicate function can be passed as an additional test on found\nneighbors.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {2, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.neighbors(grid, {1, 1}, fn {_coord, value} -> value == :mine end)\n[{{2, 1}, :mine}]\n```","ref":"Hold.CoordinateGrid.html#neighbors/3","title":"Hold.CoordinateGrid.neighbors/3","type":"function"},{"doc":"Check if `coord` has any neighbors who are members of `grid`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.neighbors?(grid, {1, 1})\ntrue\n```\n\nAn optional predicate function can be passed as an additional test on\nneighbors.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.neighbors?(grid, {1, 1}, fn {_coord, value} -> value == :mine end)\nfalse\n```","ref":"Hold.CoordinateGrid.html#neighbors?/3","title":"Hold.CoordinateGrid.neighbors?/3","type":"function"},{"doc":"Creates a new `CoordinateGrid`.\n\n```elixir\niex> Hold.CoordinateGrid.new()\n%Hold.CoordinateGrid{g: %{}}\n```","ref":"Hold.CoordinateGrid.html#new/0","title":"Hold.CoordinateGrid.new/0","type":"function"},{"doc":"Returns whether `grid` contains a coord to the right from `coord`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.right(grid, {1, 1})\n{{1, 2}, :powerup}\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.right(grid, {1, 2})\nnil\n```","ref":"Hold.CoordinateGrid.html#right/2","title":"Hold.CoordinateGrid.right/2","type":"function"},{"doc":"Returns whether `grid` contains a coord to the right of `coord`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.right?(grid, {1, 1})\ntrue\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.right?(grid, {1, 2})\nfalse\n```\n\nAn optional predicate can be passed to further test found `t:cell`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.right?(grid, {1, 1}, fn {_coord, value} -> value == :powerup end)\ntrue\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 2}, :powerup)\niex> Hold.CoordinateGrid.right?(grid, {1, 1}, fn {_coord, value} -> value == :mine end)\nfalse\n```","ref":"Hold.CoordinateGrid.html#right?/3","title":"Hold.CoordinateGrid.right?/3","type":"function"},{"doc":"Traverse over grid starting at `coord` collecting an accumulator value.\n\n`iterator` function should return either `{:cont, next_coordinate, acc}` to\nstep to next coordinate, or `{:halt, acc}` to return accumulator and stop\ntraversal.","ref":"Hold.CoordinateGrid.html#traverse/4","title":"Hold.CoordinateGrid.traverse/4","type":"function"},{"doc":"Get cell upward of `coord` in `grid`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {2, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :powerup)\niex> Hold.CoordinateGrid.up(grid, {1, 1})\n{{2, 1}, :mine}\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {2, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :powerup)\niex> Hold.CoordinateGrid.up(grid, {2, 1})\nnil\n```","ref":"Hold.CoordinateGrid.html#up/2","title":"Hold.CoordinateGrid.up/2","type":"function"},{"doc":"Returns whether `grid` contains a coord to the upward from `coord`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {2, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :powerup)\niex> Hold.CoordinateGrid.up?(grid, {1, 1})\ntrue\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {2, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :powerup)\niex> Hold.CoordinateGrid.up?(grid, {2, 1})\nfalse\n```\n\nAn optional predicate can be passed to further test found `t:cell`.\n\n```elixir\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {2, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :powerup)\niex> Hold.CoordinateGrid.up?(grid, {1, 1}, fn {_coord, value} -> value == :mine end)\ntrue\n\niex> grid = Hold.CoordinateGrid.new()\niex> grid = Hold.CoordinateGrid.add(grid, {2, 1}, :mine)\niex> grid = Hold.CoordinateGrid.add(grid, {1, 1}, :powerup)\niex> Hold.CoordinateGrid.up?(grid, {1, 1}, fn {_coord, value} -> value != :mine end)\nfalse\n```","ref":"Hold.CoordinateGrid.html#up?/3","title":"Hold.CoordinateGrid.up?/3","type":"function"},{"doc":"A representation of a valid `t:Hold.Coordinate.t/` in `grid` and `t:value/0` it is\nholding.","ref":"Hold.CoordinateGrid.html#t:cell/0","title":"Hold.CoordinateGrid.cell/0","type":"type"},{"doc":"","ref":"Hold.CoordinateGrid.html#t:member/0","title":"Hold.CoordinateGrid.member/0","type":"opaque"},{"doc":"Representation of item at a `t:Hold.Coordinate.t/0` in `t:t/0`.","ref":"Hold.CoordinateGrid.html#t:member/1","title":"Hold.CoordinateGrid.member/1","type":"opaque"},{"doc":"","ref":"Hold.CoordinateGrid.html#t:predicate/0","title":"Hold.CoordinateGrid.predicate/0","type":"opaque"},{"doc":"","ref":"Hold.CoordinateGrid.html#t:t/0","title":"Hold.CoordinateGrid.t/0","type":"type"},{"doc":"","ref":"Hold.CoordinateGrid.html#t:t/1","title":"Hold.CoordinateGrid.t/1","type":"type"},{"doc":"User provided value being stored at `t:Hold.Corrdinate.t/0` in grid.","ref":"Hold.CoordinateGrid.html#t:value/0","title":"Hold.CoordinateGrid.value/0","type":"type"},{"doc":"","ref":"Hold.Zipper.html","title":"Hold.Zipper","type":"module"},{"doc":"Error when an invalid move would have been performed.\n\nOne requirement of a zipper is that you always must be able be able to\nmaintain a focus on an element. Any moves that would result in the focus\nlooking at nothing will result in this error. Example: Moving right when no\nright sibling exists","ref":"Hold.Zipper.html#t:invalid_move/0","title":"Hold.Zipper.invalid_move/0","type":"type"},{"doc":"An implementation of a Zipper over a list\n\nIntroductory Article: https://ferd.ca/yet-another-article-on-zippers.html\n\nSupports being able to traverse a list both forwards and backwards while\nmaintaining a focus on the current element much like a doubly-linked list.\nAll while using functional immutable data structures and supporting O(1)\nlookup and traversal from focus.\n\nSo given the list `[1, 2, 3, 4, 5]` we can imagine being able to \"stop\" on 3\nand then backtrack or keep progressing forward.\n\n```\n    (3)\n  2     4\n1         5\n```\n\nZipper.List also supports θ(1) updating the currently focused item.","ref":"Hold.Zipper.List.html","title":"Hold.Zipper.List","type":"module"},{"doc":"Return zipper with focus on the first element.\n\n```elixir\niex> zipper = Hold.Zipper.List.from_list([1, 2, 3, 4, 5])\niex> {:ok, zipper} = Hold.Zipper.List.right(zipper)\niex> {:ok, zipper} = Hold.Zipper.List.right(zipper)\niex> zipper = Hold.Zipper.List.first(zipper)\niex> Hold.Zipper.List.focus(zipper)\n1\n```","ref":"Hold.Zipper.List.html#first/1","title":"Hold.Zipper.List.first/1","type":"function"},{"doc":"Return currently focused element.\n\n```elixir\niex> zipper = Hold.Zipper.List.from_list([1, 2, 3, 4, 5])\niex> Hold.Zipper.List.focus(zipper)\n1\n\niex> zipper = Hold.Zipper.List.from_list([1, 2, 3, 4, 5])\niex> {:ok, zipper} = Hold.Zipper.List.right(zipper)\niex> Hold.Zipper.List.focus(zipper)\n2\n```","ref":"Hold.Zipper.List.html#focus/1","title":"Hold.Zipper.List.focus/1","type":"function"},{"doc":"","ref":"Hold.Zipper.List.html#from_list/1","title":"Hold.Zipper.List.from_list/1","type":"function"},{"doc":"Return zipper with focus on the last element.\n\n```elixir\niex> zipper = Hold.Zipper.List.from_list([1, 2, 3, 4, 5])\niex> zipper = Hold.Zipper.List.last(zipper)\niex> Hold.Zipper.List.focus(zipper)\n5\n```","ref":"Hold.Zipper.List.html#last/1","title":"Hold.Zipper.List.last/1","type":"function"},{"doc":"Move focus left, if possible.\n\n```elixir\niex> zipper = Hold.Zipper.List.from_list([1, 2, 3, 4, 5])\niex> {:ok, zipper} = Hold.Zipper.List.right(zipper)\niex> {:ok, zipper} = Hold.Zipper.List.left(zipper)\niex> Hold.Zipper.List.focus(zipper)\n1\n\niex> zipper = Hold.Zipper.List.from_list([1, 2, 3, 4, 5])\niex> Hold.Zipper.List.left(zipper)\n{:error, :invalid_move}\n```","ref":"Hold.Zipper.List.html#left/1","title":"Hold.Zipper.List.left/1","type":"function"},{"doc":"Return whether an element to the right exists.\n\n```elixir\niex> zipper = Hold.Zipper.List.from_list([1, 2, 3, 4, 5])\niex> {:ok, zipper} = Hold.Zipper.List.right(zipper)\niex> Hold.Zipper.List.left?(zipper)\ntrue\n\niex> zipper = Hold.Zipper.List.from_list([1, 2, 3, 4, 5])\niex> Hold.Zipper.List.left?(zipper)\nfalse\n```","ref":"Hold.Zipper.List.html#left?/1","title":"Hold.Zipper.List.left?/1","type":"function"},{"doc":"Move focus right, if possible.\n\n```elixir\niex> zipper = Hold.Zipper.List.from_list([1, 2, 3, 4, 5])\niex> {:ok, zipper} = Hold.Zipper.List.right(zipper)\niex> Hold.Zipper.List.focus(zipper)\n2\n\niex> zipper = Hold.Zipper.List.from_list([1])\niex> Hold.Zipper.List.right(zipper)\n{:error, :invalid_move}\n```","ref":"Hold.Zipper.List.html#right/1","title":"Hold.Zipper.List.right/1","type":"function"},{"doc":"Return whether an element to the right exists.\n\n```elixir\niex> zipper = Hold.Zipper.List.from_list([1, 2, 3, 4, 5])\niex> Hold.Zipper.List.right?(zipper)\ntrue\n\niex> zipper = Hold.Zipper.List.from_list([1])\niex> Hold.Zipper.List.right?(zipper)\nfalse\n```","ref":"Hold.Zipper.List.html#right?/1","title":"Hold.Zipper.List.right?/1","type":"function"},{"doc":"","ref":"Hold.Zipper.List.html#to_list/1","title":"Hold.Zipper.List.to_list/1","type":"function"},{"doc":"Updates the element which is currently focused.\n\n```elixir\niex> zipper = Hold.Zipper.List.from_list([1, 2, 3, 4, 5])\niex> {:ok, zipper} = Hold.Zipper.List.right(zipper)\niex> zipper = Hold.Zipper.List.update(zipper, 20)\niex> Hold.Zipper.List.to_list(zipper)\n[1, 20, 3, 4, 5]\n```","ref":"Hold.Zipper.List.html#update/2","title":"Hold.Zipper.List.update/2","type":"function"},{"doc":"","ref":"Hold.Zipper.List.html#t:invalid_move/0","title":"Hold.Zipper.List.invalid_move/0","type":"type"},{"doc":"","ref":"Hold.Zipper.List.html#t:t/0","title":"Hold.Zipper.List.t/0","type":"type"},{"doc":"`left` represents items to the left of the focus. `right` is a list where the\nhead of it is the current focus and the rest is the items to the right of\nfocus.","ref":"Hold.Zipper.List.html#t:t/1","title":"Hold.Zipper.List.t/1","type":"type"},{"doc":"Zipper over a General Tree (any number of child nodes).\n\nIntro on Zippers: https://ferd.ca/yet-another-article-on-zippers.html","ref":"Hold.Zipper.Tree.html","title":"Hold.Zipper.Tree","type":"module"},{"doc":"Goes down one level to the children of current focus.","ref":"Hold.Zipper.Tree.html#children/1","title":"Hold.Zipper.Tree.children/1","type":"function"},{"doc":"```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> {:ok, tree} = Hold.Zipper.Tree.children(tree)\niex> Hold.Zipper.Tree.focus(tree)\n1\n```","ref":"Hold.Zipper.Tree.html#children/1-exmaples","title":"Exmaples - Hold.Zipper.Tree.children/1","type":"function"},{"doc":"Same as children/1 but will throw if not able to make move","ref":"Hold.Zipper.Tree.html#children!/1","title":"Hold.Zipper.Tree.children!/1","type":"function"},{"doc":"Returns whether current focus has any children","ref":"Hold.Zipper.Tree.html#children?/1","title":"Hold.Zipper.Tree.children?/1","type":"function"},{"doc":"```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> Hold.Zipper.Tree.children?(tree)\nfalse\n\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> Hold.Zipper.Tree.children?(tree)\ntrue\n```","ref":"Hold.Zipper.Tree.html#children?/1-examples","title":"Examples - Hold.Zipper.Tree.children?/1","type":"function"},{"doc":"Returns tree rooted at element based on `predicate` function - returning nil\nwhen no element returns true for predicate.\n\nThe `predicate` function will be invoked with the tree at the position of the\ncurrent element and the current element. The tree argument can be used if\nthere is a need to look at surrounding elements of the current focus in order\nfor the `predicate` to make a decision.","ref":"Hold.Zipper.Tree.html#find_subtree/2","title":"Hold.Zipper.Tree.find_subtree/2","type":"function"},{"doc":"Returns the currently focused element of the tree.","ref":"Hold.Zipper.Tree.html#focus/1","title":"Hold.Zipper.Tree.focus/1","type":"function"},{"doc":"```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> Hold.Zipper.Tree.focus(tree)\n0\n\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> Hold.Zipper.Tree.focus(tree)\n1 \n```","ref":"Hold.Zipper.Tree.html#focus/1-examples","title":"Examples - Hold.Zipper.Tree.focus/1","type":"function"},{"doc":"Adds a new node at the current focus with `value`.","ref":"Hold.Zipper.Tree.html#insert/2","title":"Hold.Zipper.Tree.insert/2","type":"function"},{"doc":"Inserted value becomes focus\n\n```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> tree = Hold.Zipper.Tree.insert(tree, 2)\niex> Hold.Zipper.Tree.focus(tree)\n2\n```\n\nOld focus is now the right sibling\n\n```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> tree = Hold.Zipper.Tree.insert(tree, 2)\niex> tree = Hold.Zipper.Tree.right!(tree)\niex> Hold.Zipper.Tree.focus(tree)\n1\n```","ref":"Hold.Zipper.Tree.html#insert/2-examples","title":"Examples - Hold.Zipper.Tree.insert/2","type":"function"},{"doc":"Adds a new child node to the current focus with `value`.\n\nIf the parent were to then move into the child collection they would be\nfocused on the new child.","ref":"Hold.Zipper.Tree.html#insert_child/2","title":"Hold.Zipper.Tree.insert_child/2","type":"function"},{"doc":"Focus stays at parent.\n\n```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> Hold.Zipper.Tree.focus(tree)\n0\n```\n\nChild is inserted.\n\n```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> Hold.Zipper.Tree.focus(tree)\n1\n```\n\nChild goes to the head of the right sibling list.\n\n```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 2)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> tree = Hold.Zipper.Tree.right!(tree) # Now 2 is in left siblings\niex> tree = Hold.Zipper.Tree.parent!(tree)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> tree = Hold.Zipper.Tree.left!(tree)\niex> Hold.Zipper.Tree.focus(tree)\n2\n```","ref":"Hold.Zipper.Tree.html#insert_child/2-examples","title":"Examples - Hold.Zipper.Tree.insert_child/2","type":"function"},{"doc":"Adds a new node to the right of current focus with `value`.\n\nDoes not shift focus to new node.","ref":"Hold.Zipper.Tree.html#insert_right/2","title":"Hold.Zipper.Tree.insert_right/2","type":"function"},{"doc":"```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> tree = Hold.Zipper.Tree.insert_right(tree, 2)\niex> Hold.Zipper.Tree.focus(tree)\n1\n\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> tree = Hold.Zipper.Tree.insert_right(tree, 2)\niex> tree = Hold.Zipper.Tree.right!(tree)\niex> Hold.Zipper.Tree.focus(tree)\n2\n```","ref":"Hold.Zipper.Tree.html#insert_right/2-examples","title":"Examples - Hold.Zipper.Tree.insert_right/2","type":"function"},{"doc":"","ref":"Hold.Zipper.Tree.html#insert_right/2-examples","title":"Examples - Hold.Zipper.Tree.insert_right/2","type":"function"},{"doc":"Moves focus to the left of the current level","ref":"Hold.Zipper.Tree.html#left/1","title":"Hold.Zipper.Tree.left/1","type":"function"},{"doc":"```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 2)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> tree = Hold.Zipper.Tree.right!(tree)\niex> {:ok, tree} = Hold.Zipper.Tree.left(tree)\niex> Hold.Zipper.Tree.focus(tree)\n1\n```","ref":"Hold.Zipper.Tree.html#left/1-examples","title":"Examples - Hold.Zipper.Tree.left/1","type":"function"},{"doc":"Same as left/1 but will throw if not able to make move","ref":"Hold.Zipper.Tree.html#left!/1","title":"Hold.Zipper.Tree.left!/1","type":"function"},{"doc":"Returns whether a left siblings exists","ref":"Hold.Zipper.Tree.html#left?/1","title":"Hold.Zipper.Tree.left?/1","type":"function"},{"doc":"```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> Hold.Zipper.Tree.left?(tree)\nfalse\n\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 2)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> tree = Hold.Zipper.Tree.right!(tree)\niex> Hold.Zipper.Tree.left?(tree)\ntrue\n```","ref":"Hold.Zipper.Tree.html#left?/1-examples","title":"Examples - Hold.Zipper.Tree.left?/1","type":"function"},{"doc":"Creates an iterator over a tree walking over all of the elements until\ncompletion.\n\nEach iteration will return `{:ok, item, new_tree}` until the tree has been\nfully visited. At which point `{:complete, last_item, new_tree}` will be\nreturned","ref":"Hold.Zipper.Tree.html#next/1","title":"Hold.Zipper.Tree.next/1","type":"function"},{"doc":"Moves up to direct parent of current focus leaving current siblings intact.\n\nThe current level siblings list will not be reset before moving up. For\nexample if you were on the second sibling of the current level moving up with\nparent/1 and then calling children/1 would move you back down to the second\nsibling again.","ref":"Hold.Zipper.Tree.html#parent/1","title":"Hold.Zipper.Tree.parent/1","type":"function"},{"doc":"```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> {:ok, tree} = Hold.Zipper.Tree.parent(tree)\niex> Hold.Zipper.Tree.focus(tree)\n0\n```\n\nMoving to parent doesn't reset left and right sibling list.\n\n```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 2)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> tree = Hold.Zipper.Tree.right!(tree) # 1 is now in left sibling list\niex> {:ok, tree} = Hold.Zipper.Tree.parent(tree)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> Hold.Zipper.Tree.focus(tree)\n2\n```","ref":"Hold.Zipper.Tree.html#parent/1-examples","title":"Examples - Hold.Zipper.Tree.parent/1","type":"function"},{"doc":"Same as parent/1 but will throw if not able to make move","ref":"Hold.Zipper.Tree.html#parent!/1","title":"Hold.Zipper.Tree.parent!/1","type":"function"},{"doc":"","ref":"Hold.Zipper.Tree.html#parent?/1","title":"Hold.Zipper.Tree.parent?/1","type":"function"},{"doc":"Moves focus to the right of the current level","ref":"Hold.Zipper.Tree.html#right/1","title":"Hold.Zipper.Tree.right/1","type":"function"},{"doc":"```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 2)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> {:ok, tree} = Hold.Zipper.Tree.right(tree)\niex> Hold.Zipper.Tree.focus(tree)\n2\n```","ref":"Hold.Zipper.Tree.html#right/1-examples","title":"Examples - Hold.Zipper.Tree.right/1","type":"function"},{"doc":"Same as right/1 but will throw if not able to make move","ref":"Hold.Zipper.Tree.html#right!/1","title":"Hold.Zipper.Tree.right!/1","type":"function"},{"doc":"Returns whether a right sibling exists.","ref":"Hold.Zipper.Tree.html#right?/1","title":"Hold.Zipper.Tree.right?/1","type":"function"},{"doc":"```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> Hold.Zipper.Tree.right?(tree)\nfalse\n\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 2)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> Hold.Zipper.Tree.right?(tree)\ntrue\n```","ref":"Hold.Zipper.Tree.html#right?/1-examples","title":"Examples - Hold.Zipper.Tree.right?/1","type":"function"},{"doc":"Creates a new tree with `value` placed as root.","ref":"Hold.Zipper.Tree.html#root/1","title":"Hold.Zipper.Tree.root/1","type":"function"},{"doc":"```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> Hold.Zipper.Tree.focus(tree)\n0\n```","ref":"Hold.Zipper.Tree.html#root/1-examples","title":"Examples - Hold.Zipper.Tree.root/1","type":"function"},{"doc":"Returns whether current focus is the root node","ref":"Hold.Zipper.Tree.html#root?/1","title":"Hold.Zipper.Tree.root?/1","type":"function"},{"doc":"```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> Hold.Zipper.Tree.root?(tree)\ntrue\n\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> Hold.Zipper.Tree.root?(tree)\nfalse\n```","ref":"Hold.Zipper.Tree.html#root?/1-examples","title":"Examples - Hold.Zipper.Tree.root?/1","type":"function"},{"doc":"Moves up to parent similar to parent/1, but also resets current level\n\nIf for example you were on the second sibling of the current level moivng up\nwith rparent/1 and then calling children/1 would move you back down to\nfirst sibling instead of the second.","ref":"Hold.Zipper.Tree.html#rparent/1","title":"Hold.Zipper.Tree.rparent/1","type":"function"},{"doc":"```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> {:ok, tree} = Hold.Zipper.Tree.parent(tree)\niex> Hold.Zipper.Tree.focus(tree)\n0\n```\n\nUnlike `parent/1` the left and right sibling list is reset\n\n```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 2)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> tree = Hold.Zipper.Tree.right!(tree) # 1 is now in left sibling list\niex> {:ok, tree} = Hold.Zipper.Tree.rparent(tree) # This resets sibling list\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> Hold.Zipper.Tree.focus(tree)\n1\n```","ref":"Hold.Zipper.Tree.html#rparent/1-examples","title":"Examples - Hold.Zipper.Tree.rparent/1","type":"function"},{"doc":"Same as rparent/1 but will throw if not able to make move","ref":"Hold.Zipper.Tree.html#rparent!/1","title":"Hold.Zipper.Tree.rparent!/1","type":"function"},{"doc":"Updates the value currently at focus with `value`.\n\nThe new value at node will keep the children present in the old node.","ref":"Hold.Zipper.Tree.html#update/2","title":"Hold.Zipper.Tree.update/2","type":"function"},{"doc":"Update the current focus.\n\n```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.update(tree, 0.5)\niex> Hold.Zipper.Tree.focus(tree)\n0.5\n```\n\nChildren of the updated node do not change.\n\n```elixir\niex> tree = Hold.Zipper.Tree.root(0)\niex> tree = Hold.Zipper.Tree.insert_child(tree, 1)\niex> tree = Hold.Zipper.Tree.update(tree, 0.5)\niex> tree = Hold.Zipper.Tree.children!(tree)\niex> Hold.Zipper.Tree.focus(tree)\n1\n```","ref":"Hold.Zipper.Tree.html#update/2-examples","title":"Examples - Hold.Zipper.Tree.update/2","type":"function"},{"doc":"","ref":"Hold.Zipper.Tree.html#t:t/0","title":"Hold.Zipper.Tree.t/0","type":"type"},{"doc":"","ref":"Hold.Zipper.Tree.html#t:t/1","title":"Hold.Zipper.Tree.t/1","type":"type"},{"doc":"A thread is a list of znodes behind us in the traversal. They support being\nable to walk backwards the way we came.","ref":"Hold.Zipper.Tree.html#t:thread/0","title":"Hold.Zipper.Tree.thread/0","type":"type"},{"doc":"","ref":"Hold.Zipper.Tree.html#t:thread/1","title":"Hold.Zipper.Tree.thread/1","type":"type"},{"doc":"Znodes represent levels of the tree as a Zipper List.\n\nGiven this tree:\n\n        A\n      ↙   ↘ \n     B     C\n\nIf the focus was at B the resulting znode would be.\n\n{\n  # left siblings\n  [],\n  # current focus | right siblings\n  [\n    {\n      B,\n      nil\n    },\n    {\n      C,\n      nil\n    },\n  ]\n}\n\nThe focus of the zipper is a 2 element tuple. The first item represents the\ncurrently focused node. the second element represents the focused nodes\nchildren. `nil` shows that it has no children and a znode when it has them.","ref":"Hold.Zipper.Tree.html#t:znode/0","title":"Hold.Zipper.Tree.znode/0","type":"type"},{"doc":"","ref":"Hold.Zipper.Tree.html#t:znode/1","title":"Hold.Zipper.Tree.znode/1","type":"type"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}